using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using NationalInstruments.DAQmx;

namespace accel_compass
{
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class FormCalibrate : System.Windows.Forms.Form
	{
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.NumericUpDown numericNumSamples;
		private System.Windows.Forms.Label accelRate;
		private System.Windows.Forms.NumericUpDown numericAccelRate;
		private System.Windows.Forms.Button buttonStartCalib;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.Label labelAverageX;
		private System.Windows.Forms.Label label3;
		private System.Windows.Forms.Label labelStdDevX;
		private System.Windows.Forms.Button buttonAccept;
		private System.Windows.Forms.Label label4;
		private System.Windows.Forms.Label label5;
		private System.Windows.Forms.Label labelStdDevY;
		private System.Windows.Forms.Label labelAverageY;
		private System.Windows.Forms.Label label8;
		private System.Windows.Forms.Label labelSkewY;
		private System.Windows.Forms.Label labelSkewX;


		private Task taskDevice;

		private AnalogMultiChannelReader readerDevice;


		private double _averageAccel1;
		private double _averageAccel2;
		private double _stdDevAccel1;
		private double _stdDevAccel2;
		private double _skewAccel1;
		private double _skewAccel2;

		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public FormCalibrate()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			//Create the channels for analog input:

			taskDevice = new Task();


			// Initialize local variables
			double sampleRate = Convert.ToDouble(numericAccelRate.Value);
			int samplesPerChannel = Convert.ToInt32(numericNumSamples.Value);
			
			//Create a virtual channel
			taskDevice.AIChannels.CreateVoltageChannel(Robot.getInstance().getAccelDevice1() ,"",
				AITerminalConfiguration.Rse,0,
				5,AIVoltageUnits.Volts);  

			taskDevice.AIChannels.CreateVoltageChannel(Robot.getInstance().getAccelDevice2() ,"",
				AITerminalConfiguration.Rse,0,
				5,AIVoltageUnits.Volts);  



			// Configure timing specs    
			taskDevice.Timing.ConfigureSampleClock("", sampleRate, SampleClockActiveEdge.Rising,
				SampleQuantityMode.FiniteSamples, samplesPerChannel);



			readerDevice = new AnalogMultiChannelReader(taskDevice.Stream);

			//Verify the Task

			taskDevice.Control(TaskAction.Verify);



#if NETFX2_0
                // For .NET Framework 2.0, use SynchronizeCallbacks to specify that the object 
                // marshals callbacks across threads appropriately.
                readerDevice.SynchronizeCallbacks = true;
#else
			// For .NET Framework 1.1, set SynchronizingObject to the Windows Form to specify 
			// that the object marshals callbacks across threads appropriately.
			readerDevice.SynchronizingObject = this;
#endif

		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.label1 = new System.Windows.Forms.Label();
			this.numericNumSamples = new System.Windows.Forms.NumericUpDown();
			this.buttonStartCalib = new System.Windows.Forms.Button();
			this.label2 = new System.Windows.Forms.Label();
			this.labelAverageX = new System.Windows.Forms.Label();
			this.label3 = new System.Windows.Forms.Label();
			this.labelStdDevX = new System.Windows.Forms.Label();
			this.buttonAccept = new System.Windows.Forms.Button();
			this.label4 = new System.Windows.Forms.Label();
			this.label5 = new System.Windows.Forms.Label();
			this.labelStdDevY = new System.Windows.Forms.Label();
			this.labelAverageY = new System.Windows.Forms.Label();
			this.accelRate = new System.Windows.Forms.Label();
			this.numericAccelRate = new System.Windows.Forms.NumericUpDown();
			this.labelSkewY = new System.Windows.Forms.Label();
			this.labelSkewX = new System.Windows.Forms.Label();
			this.label8 = new System.Windows.Forms.Label();
			((System.ComponentModel.ISupportInitialize)(this.numericNumSamples)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.numericAccelRate)).BeginInit();
			this.SuspendLayout();
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(80, 26);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(120, 24);
			this.label1.TabIndex = 0;
			this.label1.Text = "Number of Samples";
			// 
			// numericNumSamples
			// 
			this.numericNumSamples.Increment = new System.Decimal(new int[] {
																				500,
																				0,
																				0,
																				0});
			this.numericNumSamples.Location = new System.Drawing.Point(208, 26);
			this.numericNumSamples.Maximum = new System.Decimal(new int[] {
																			  50000,
																			  0,
																			  0,
																			  0});
			this.numericNumSamples.Minimum = new System.Decimal(new int[] {
																			  1,
																			  0,
																			  0,
																			  0});
			this.numericNumSamples.Name = "numericNumSamples";
			this.numericNumSamples.Size = new System.Drawing.Size(72, 21);
			this.numericNumSamples.TabIndex = 1;
			this.numericNumSamples.Value = new System.Decimal(new int[] {
																			1000,
																			0,
																			0,
																			0});
			this.numericNumSamples.ValueChanged += new System.EventHandler(this.numericNumSamples_ValueChanged);
			// 
			// buttonStartCalib
			// 
			this.buttonStartCalib.Location = new System.Drawing.Point(104, 103);
			this.buttonStartCalib.Name = "buttonStartCalib";
			this.buttonStartCalib.Size = new System.Drawing.Size(168, 32);
			this.buttonStartCalib.TabIndex = 2;
			this.buttonStartCalib.Text = "Start Calibration";
			this.buttonStartCalib.Click += new System.EventHandler(this.buttonStartCalib_Click);
			// 
			// label2
			// 
			this.label2.Location = new System.Drawing.Point(32, 168);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(96, 24);
			this.label2.TabIndex = 3;
			this.label2.Text = "Average:";
			// 
			// labelAverageX
			// 
			this.labelAverageX.Location = new System.Drawing.Point(144, 172);
			this.labelAverageX.Name = "labelAverageX";
			this.labelAverageX.Size = new System.Drawing.Size(80, 24);
			this.labelAverageX.TabIndex = 4;
			this.labelAverageX.Text = "---";
			// 
			// label3
			// 
			this.label3.Location = new System.Drawing.Point(32, 192);
			this.label3.Name = "label3";
			this.label3.Size = new System.Drawing.Size(80, 23);
			this.label3.TabIndex = 5;
			this.label3.Text = "Std Dev.";
			// 
			// labelStdDevX
			// 
			this.labelStdDevX.Location = new System.Drawing.Point(144, 192);
			this.labelStdDevX.Name = "labelStdDevX";
			this.labelStdDevX.Size = new System.Drawing.Size(80, 24);
			this.labelStdDevX.TabIndex = 6;
			this.labelStdDevX.Text = "---";
			// 
			// buttonAccept
			// 
			this.buttonAccept.Enabled = false;
			this.buttonAccept.Location = new System.Drawing.Point(136, 250);
			this.buttonAccept.Name = "buttonAccept";
			this.buttonAccept.Size = new System.Drawing.Size(104, 32);
			this.buttonAccept.TabIndex = 7;
			this.buttonAccept.Text = "Accept Values";
			this.buttonAccept.Click += new System.EventHandler(this.buttonAccept_Click);
			// 
			// label4
			// 
			this.label4.Location = new System.Drawing.Point(136, 146);
			this.label4.Name = "label4";
			this.label4.Size = new System.Drawing.Size(54, 24);
			this.label4.TabIndex = 8;
			this.label4.Text = "X-Axis";
			// 
			// label5
			// 
			this.label5.Location = new System.Drawing.Point(248, 146);
			this.label5.Name = "label5";
			this.label5.Size = new System.Drawing.Size(54, 24);
			this.label5.TabIndex = 9;
			this.label5.Text = "Y-Axis";
			// 
			// labelStdDevY
			// 
			this.labelStdDevY.Location = new System.Drawing.Point(256, 192);
			this.labelStdDevY.Name = "labelStdDevY";
			this.labelStdDevY.Size = new System.Drawing.Size(88, 24);
			this.labelStdDevY.TabIndex = 11;
			this.labelStdDevY.Text = "---";
			// 
			// labelAverageY
			// 
			this.labelAverageY.Location = new System.Drawing.Point(256, 172);
			this.labelAverageY.Name = "labelAverageY";
			this.labelAverageY.Size = new System.Drawing.Size(88, 24);
			this.labelAverageY.TabIndex = 10;
			this.labelAverageY.Text = "---";
			// 
			// accelRate
			// 
			this.accelRate.Location = new System.Drawing.Point(80, 60);
			this.accelRate.Name = "accelRate";
			this.accelRate.Size = new System.Drawing.Size(120, 24);
			this.accelRate.TabIndex = 12;
			this.accelRate.Text = "Rate (Hz)";
			// 
			// numericAccelRate
			// 
			this.numericAccelRate.Increment = new System.Decimal(new int[] {
																			   100,
																			   0,
																			   0,
																			   0});
			this.numericAccelRate.Location = new System.Drawing.Point(208, 60);
			this.numericAccelRate.Maximum = new System.Decimal(new int[] {
																			 5000,
																			 0,
																			 0,
																			 0});
			this.numericAccelRate.Minimum = new System.Decimal(new int[] {
																			 1,
																			 0,
																			 0,
																			 0});
			this.numericAccelRate.Name = "numericAccelRate";
			this.numericAccelRate.Size = new System.Drawing.Size(72, 21);
			this.numericAccelRate.TabIndex = 13;
			this.numericAccelRate.Value = new System.Decimal(new int[] {
																		   5000,
																		   0,
																		   0,
																		   0});
			this.numericAccelRate.ValueChanged += new System.EventHandler(this.numericAccelRate_ValueChanged);
			// 
			// labelSkewY
			// 
			this.labelSkewY.Location = new System.Drawing.Point(256, 216);
			this.labelSkewY.Name = "labelSkewY";
			this.labelSkewY.Size = new System.Drawing.Size(88, 24);
			this.labelSkewY.TabIndex = 16;
			this.labelSkewY.Text = "---";

			// 
			// labelSkewX
			// 
			this.labelSkewX.Location = new System.Drawing.Point(144, 216);
			this.labelSkewX.Name = "labelSkewX";
			this.labelSkewX.Size = new System.Drawing.Size(80, 24);
			this.labelSkewX.TabIndex = 15;
			this.labelSkewX.Text = "---";
			// 
			// label8
			// 
			this.label8.Location = new System.Drawing.Point(32, 216);
			this.label8.Name = "label8";
			this.label8.Size = new System.Drawing.Size(80, 23);
			this.label8.TabIndex = 14;
			this.label8.Text = "Skewness";
			// 
			// FormCalibrate
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 14);
			this.ClientSize = new System.Drawing.Size(362, 301);
			this.Controls.Add(this.labelSkewY);
			this.Controls.Add(this.labelSkewX);
			this.Controls.Add(this.label8);
			this.Controls.Add(this.numericAccelRate);
			this.Controls.Add(this.accelRate);
			this.Controls.Add(this.labelStdDevY);
			this.Controls.Add(this.labelAverageY);
			this.Controls.Add(this.label5);
			this.Controls.Add(this.label4);
			this.Controls.Add(this.buttonAccept);
			this.Controls.Add(this.labelStdDevX);
			this.Controls.Add(this.label3);
			this.Controls.Add(this.labelAverageX);
			this.Controls.Add(this.label2);
			this.Controls.Add(this.buttonStartCalib);
			this.Controls.Add(this.numericNumSamples);
			this.Controls.Add(this.label1);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
			this.MaximizeBox = false;
			this.Name = "FormCalibrate";
			this.SizeGripStyle = System.Windows.Forms.SizeGripStyle.Hide;
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterParent;
			this.Text = "Accelerometer Calibration";
			((System.ComponentModel.ISupportInitialize)(this.numericNumSamples)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.numericAccelRate)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion

		private void buttonAccept_Click(object sender, System.EventArgs e)
		{
			Robot.getInstance().setAccel1CalibValue(_averageAccel1);
			Robot.getInstance().setAccel2CalibValue(_averageAccel2);
			Dispose(true);
		}

		private void buttonStartCalib_Click(object sender, System.EventArgs e)
		{
			buttonStartCalib.Enabled = false;
			buttonAccept.Enabled = false;

			// Initialize local variables
			
			int numSamples = Convert.ToInt32(numericNumSamples.Value);
			double[] samples1 = new double[numSamples];
			double[] samples2 = new double[numSamples];

			double [,] samples = getMultiSampleOnReader(readerDevice, numSamples);


			for(int i = 0; i < samples.GetLength(1); i++)
			{	
				samples1[i] = samples[0, i];
				samples2[i] = samples[1, i];
			}

			_averageAccel1 =  Stat.GetAverage(samples1);
			_averageAccel2 =  Stat.GetAverage(samples2);
			_stdDevAccel1 = Stat.GetStdDev(samples1);
			_stdDevAccel2 = Stat.GetStdDev(samples2);
			_skewAccel1 = Stat.GetSkewness(samples1);
			_skewAccel2 = Stat.GetSkewness(samples2);

			//MessageBox.Show("Device1 average: " + Convert.ToString(average1) + "\n" + "Device2 average: " + Convert.ToString(average2) );

			labelAverageX.Text = Convert.ToString( Math.Round(_averageAccel1, 6) );
			labelAverageY.Text = Convert.ToString( Math.Round(_averageAccel2, 6) );

			labelStdDevX.Text = Convert.ToString( Math.Round(_stdDevAccel1, 6) );
			labelStdDevY.Text = Convert.ToString( Math.Round(_stdDevAccel2, 6) );

			labelSkewX.Text = Convert.ToString( Math.Round(_skewAccel1, 6) );
			labelSkewY.Text = Convert.ToString( Math.Round(_skewAccel2, 6) );


			buttonAccept.Enabled = true;
			buttonStartCalib.Enabled = true;

		}


		private double getSampleOnReader(AnalogMultiChannelReader reader)
		{
			double[] data;
			data = reader.ReadSingleSample(); 

			if(data.Length >= 1) 
			{
				return data[0];
			}
			else
			{
				return 0;
			}

		}



		private double [,] getMultiSampleOnReader(AnalogMultiChannelReader reader, int samplesPerChannel)
		{
			return reader.ReadMultiSample(samplesPerChannel);
		}




		//dynamic configuration :

		private void numericNumSamples_ValueChanged(object sender, System.EventArgs e)
		{
		
			double sampleRate = Convert.ToDouble(numericAccelRate.Value);
			int samplesPerChannel = Convert.ToInt32(numericNumSamples.Value);

			// Configure timing specs    
			taskDevice.Timing.ConfigureSampleClock("", sampleRate, SampleClockActiveEdge.Rising,
				SampleQuantityMode.FiniteSamples, samplesPerChannel);


		}

		private void numericAccelRate_ValueChanged(object sender, System.EventArgs e)
		{
			double sampleRate = Convert.ToDouble(numericAccelRate.Value);
			int samplesPerChannel = Convert.ToInt32(numericNumSamples.Value);

			// Configure timing specs    
			taskDevice.Timing.ConfigureSampleClock("", sampleRate, SampleClockActiveEdge.Rising,
				SampleQuantityMode.FiniteSamples, samplesPerChannel);

		}

	}
}
