using System;
using System.Threading;

namespace accel_compass
{
	/// <summary>
	/// Summary description for headingFollowerBehavior.
	/// </summary>
	public class HeadingFollowerBehavior
	{
		private bool runFlag = false;
		private bool terminateFlag = false;
		private ThreadStart threadStart = null;
		private Thread thread = null;

		private Robot _robot = null;
		private PIcontroller pic = new PIcontroller();
		private double desiredHeading = 0;

		public HeadingFollowerBehavior(Robot r)
		{
			_robot = r;
			threadStart = new ThreadStart(threadFunc);
			thread = new Thread(threadStart);
			thread.Start();
		}

		public void start()
		{
			runFlag = true;
		}

		public void stop()
		{
			runFlag = false;
			pic.reset();
		}

		public void terminate()
		{
			this.stop();
			terminateFlag = true;
		}

		private void threadFunc()
		{
			double deadGap = 1.0;

			while(!terminateFlag)
			{
				while(runFlag)
				{
					// comportamento propriamente dito
					double realHeading = _robot.getDigitalHeading();
					double headingError = 0;

					if(Math.Abs(realHeading - desiredHeading) <= 180)
					{
						headingError = realHeading - desiredHeading;
					}
					else if(realHeading > 180)
					{
						headingError = desiredHeading - realHeading + 180;
					}
					else
					{
						headingError = realHeading - desiredHeading + 360;
					}

					double output = pic.updateFilter(headingError);

					if(output > (5-deadGap))
					{
						output = 5-deadGap;
					}
					else if(output < - (5-deadGap))
					{
						output = -(5-deadGap);
					}

					if(output>0)
					{
						_robot.setLeftMotor(deadGap);
						_robot.setRightMotor(deadGap+Math.Abs(output));
					}
					else
					{
						_robot.setLeftMotor(deadGap+Math.Abs(output));
						_robot.setRightMotor(deadGap);

					}
				

					Thread.Sleep(10);
				}

				Thread.Sleep(100);
			}
		}

		public void setDesiredHeading(int h)
		{
			desiredHeading = h;
		}

	}
}
