using System;

namespace accel_compass
{
	/// <summary>
	/// Summary description for Stat.
	/// </summary>
	public class Stat
	{

		public static double GetAverage( double[] data )
		{
			int len = data.Length; 
			if ( len == 0 ) 
				throw new Exception("No data");
    
			double sum = 0; 
			for ( int i = 0; i < data.Length; i++ )
				sum += data[i]; 
			return sum / len;
		}

		public static double GetStdDev( double [] data )
		{
			int len = data.Length; 
			if ( len == 0 ) 
				throw new Exception("No data");


			double average = GetAverage(data);
			double sum = 0;
			
			for ( int i = 0; i < data.Length; i++ )
			{
				sum += Math.Pow((data[i] - average),2); 
			}

			double res = Math.Sqrt(sum/len);

			return res;
		}

		public static double GetSkewness( double [] data )
		{
			//http://en.wikipedia.org/wiki/Skewness

			int len = data.Length; 
			if ( len == 0 ) 
				throw new Exception("No data");

			
			double average = GetAverage(data);
			double sumOfErrors3 = 0;
			double sumOfErrors2 = 0;

			for ( int i = 0; i < data.Length; i++ )
			{
				sumOfErrors3 += Math.Pow((data[i] - average),3); 
				sumOfErrors2 += Math.Pow((data[i] - average),2); 
			}


			double m3 = Math.Sqrt(len)*sumOfErrors3;

			double m232 = Math.Pow(sumOfErrors2, 3.0/2.0);

			double g1 = m3/m232;

			double G1 = Math.Sqrt(len*(len-1))/(len-2) * g1;

			return G1;

		}
	}
}
